/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package FMG.Conductor.comm;

import java.util.Vector;

/**
 * The MessageQueue is a container class used for storing Message objects
 * in FIFO order.  This class is safe for use between threads but provides
 * no blocking or signalling.
 *
 * @author Mark Yarvis (yarvis@fmg.cs.ucla.edu)
 * @version $Revision: 1.4 $
 */

public class MessageQueue {
   /** The underlying storage for the data */
   Vector messages = new Vector();

   /**
    * Add a message to the tail of the queue.
    * @param m The new message.
    * @return No return value.
    */
   public synchronized void enqueue(Message m) {
      messages.addElement(m);
   }

   /**
    * Pull a message off the head of the queue.
    * @return The oldest message in the queue, or null if the queue is empty.
    */
   public synchronized Message dequeue() {
      if (messages.isEmpty()) {
         return null;
      }

      Message m = (Message) messages.firstElement();
      messages.removeElementAt(0);
      return m;
   }

   /**
    * Determine if the queue is empty.
    * @return True iff the queue is empty.
    */
   public synchronized boolean isEmpty() {
      return messages.isEmpty();
   }
}
