/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package FMG.Conductor.iface;

import java.util.*;

import FMG.Conductor.*;
import FMG.Conductor.util.Debug;

public class TimestampInterface extends Interface {

   protected Date rawTime = new Date();
   protected Calendar time = new GregorianCalendar();

   private StringBuffer sb = new StringBuffer();

   /** Timestamp interval in seconds. */
   protected static long INTERVAL = 
      Conductor.getIntProperty("TimestampInterface.Interval", 3600);

   protected static String days[] = new String[8];
   protected static String months[] = new String[13];

   static {
      days[Calendar.MONDAY]    = "Mon";
      days[Calendar.TUESDAY]   = "Tue";
      days[Calendar.WEDNESDAY] = "Wed";
      days[Calendar.THURSDAY]  = "Thu";
      days[Calendar.FRIDAY]    = "Fri";
      days[Calendar.SATURDAY]  = "Sat";
      days[Calendar.SUNDAY]    = "Sun";

      months[Calendar.JANUARY]   = "Jan";
      months[Calendar.FEBRUARY]  = "Feb";
      months[Calendar.MARCH]     = "Mar";
      months[Calendar.APRIL]     = "Apr";
      months[Calendar.MAY]       = "May";
      months[Calendar.JUNE]      = "Jun";
      months[Calendar.JULY]      = "Jul";
      months[Calendar.AUGUST]    = "Aug";
      months[Calendar.SEPTEMBER] = "Sep";
      months[Calendar.OCTOBER]   = "Oct";
      months[Calendar.NOVEMBER]  = "Nov";
      months[Calendar.DECEMBER]  = "Dec";
   }

   protected void runInterface() {
      super.runInterface();
      runTimer();
   }

   protected String twoDigits(int n) {
      if (n < 10) {
         return ("0" + n);
      } else {
         return ("" + n);
      }
   }

   protected String getTimeString() {
      rawTime.setTime(System.currentTimeMillis());
      time.setTime(rawTime);

      sb.setLength(0);

      sb.append(days[time.get(Calendar.DAY_OF_WEEK)]);
      sb.append(' ');
      sb.append(months[time.get(Calendar.MONTH)]);
      sb.append(' ');
      sb.append(time.get(Calendar.DAY_OF_MONTH));
      sb.append(' ');
      sb.append(twoDigits(time.get(Calendar.HOUR_OF_DAY)));
      sb.append(':');
      sb.append(twoDigits(time.get(Calendar.MINUTE)));
      sb.append(':');
      sb.append(twoDigits(time.get(Calendar.SECOND)));
      sb.append(' ');
      sb.append(time.get(Calendar.YEAR));

      return sb.toString();
   }

   protected void runTimer() {
      long msecInterval = INTERVAL * 1000;
      Thread myThread = Thread.currentThread();

      while (true) {
         Debug.println("======= " + getTimeString() + " =======");

         try {
            myThread.sleep(msecInterval);
         } catch (InterruptedException ignore) { 
            Debug.println(Debug.WARNING, 
                          "Warning: TimestampInterface Interrupted");
         }
      }
   }
}
