/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package FMG.Conductor.util;

import java.util.Hashtable;
import java.util.Enumeration;

/**
 * This class provides a list of items whose membership can be checked
 * with the speed of a hash lookup.  Elements should implement the
 * <code>hashCode</code> and <code>equals</code> methods.
 *
 * @author Mark Yarvis (yarvis@fmg.cs.ucla.edu)
 * @version $Revision: 1.4 $
 */

public class Hashlist {
   /** A hash table that stores the entries as keys to the table. */
   protected Hashtable table = new Hashtable();

   /** A dummy object stored in the hashtable under each key. */
   private Object dummy = new Object();

   /**
    * Add a new element to the hash list.  Elements should implement the 
    * <code>hashCode</code> and <code>equals</code> methods.
    * @param elem The new element to add.
    * @return No return value.
    */
   public void addElement(Object elem) {
      table.put(elem, dummy);
   }

   /**
    * Remove a previously added element from the list.
    * @param elem The new element to remove.
    * @return No return value.
    */
   public void removeElement(Object elem) {
      table.remove(elem);
   }

   /**
    * Determine if the given element is in the list.
    * @param elem The new element to look for.
    * @return True iff the given element is currently in the list.
    */
   public boolean isElement(Object elem) {
      return table.containsKey(elem);
   }

   /**
    * Remove all elements from the hashlist, leaving an empty hashlist.
    * @return No return value.
    */
   public void removeAllElements() {
      Enumeration e = table.elements();
      while (e.hasMoreElements()) {
         table.remove(e.nextElement());
      }
   }
}
