/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package FMG.Conductor.util;

import java.util.*;

/**
 * The StringOps class provides some basic routines for string manipulation
 * that are not provided by the String class.
 *
 * @author Mark Yarvis (yarvis@fmg.cs.ucla.edu)
 * @version $Revision: 1.3 $
 */

public abstract class StringOps {

   /**
    * Break a string into whitespace delimited tokens.
    * @param s The string to tokenize.
    * @return An array of strings that are the tokens from the input string. 
    */ 
   public static String[] tokenizeString(String s) { 
      if (s == null) {
         return null;
      }

      StringTokenizer st = new StringTokenizer(s," \t\n\r", false);
      String[] tokens = new String[st.countTokens()];
      int i = 0;
 
      while (st.hasMoreTokens()) {
         tokens[i++] = st.nextToken();
      }
      
      return tokens;
   }

   /**
    * Perform substring substitutions in a string.
    * @param s The string to operation on.
    * @param oldSubStr The substring to be replaced.
    * @param newSubStr The substring to replace oldSubStr with.
    * @return A new String object with all instances of oldSubStr replaced
    * by newSubStr.
    */   
   public static String replace(String s, String oldSubStr, String newSubStr) {
      if (s == null) {
         return null;
      }

      int index = s.indexOf(oldSubStr);
                               
      if (index == -1) {       
         return s;
      } else {
         String updated = s.substring(0, index) + newSubStr +
                          s.substring(index + oldSubStr.length(), s.length());
         // call recursively in case oldSubStr occurs more than once
         return replace(updated, oldSubStr, newSubStr);
      }
   }  

}
