/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package FMG.ConductorExt.adaptors.general;

import FMG.Conductor.dataflow.*;
import FMG.ConductorExt.adaptors.tools.*;
import FMG.Conductor.util.Debug;

/**
 * Give a stream the highest possible priority.  Also, make lower priority
 * streams wait 10 seconds before proceeding (to see if I have more write).
 * <p>
 * This adaptor works independently of all other adaptations.  This adaptor
 * should be the last adaptor on a given node.
 * <p>
 * @author Mark Yarvis (yarvis@fmg.cs.ucla.edu)
 * @version $Revision: 1.6 $      
 */

public class Prioritizer extends Adaptor {

   PriorityWriter priorityWriter;

   public Prioritizer(AdaptorWindow window, String parameter) {
      super(window, parameter);

      priorityWriter = new PriorityWriter(window, Integer.MAX_VALUE, 10000, 
                                          getNodeIAC());
   }

   public void handleFlow() {
      window.requestDownstreamBuffering(AdaptorWindow.MIN_BUFFERING);

      while (true) {
         try {
            priorityWriter.writeData(Integer.MAX_VALUE);
         } catch (OutputClosedException e) {
            Debug.println(Debug.ADAPTOR, Debug.INFO,
                          this + ": output closed unexpectedly");
            return;
         } catch (InputClosedException e) {
            if (Debug.on) {
               Debug.print(Debug.ADAPTOR, Debug.INFO, ")");
            }
            window.contract();
            window.closeOutput();
            Debug.println(Debug.ADAPTOR, Debug.INFO,
                          this +": closed");
            return;
         } catch (PlanChangedException e) {
            throw new RuntimeException("Case not handled.");
         } catch (AsyncEventException e) {
             Debug.println(Debug.ADAPTOR, Debug.WARNING,
                           this + ": Got an unexpected exception");
             return;
         }
      }
   }

}
