/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package FMG.ConductorExt.adaptors.http;

import java.util.Enumeration;
import java.io.*;
import java.awt.*;
import java.awt.font.*;
import java.awt.geom.*;
import java.awt.image.*;
import com.sun.image.codec.jpeg.*;
import java.awt.image.BufferedImage;
import javax.swing.*;
import FMG.Conductor.dataflow.*;
import FMG.Conductor.util.*;

import FMG.ConductorExt.adaptors.tools.*;

/**
 * This adaptor takes in an ascii text feed and re-encodes it as a
 * jpeg image.
 * @author Kevin Eustice (kfe@cs.ucla.edu)
 * @version $Revision: 1.5 $      
 */
public class AsciiJpeg extends HttpResponseAdaptor {

  public static final String ORIG_ENCODING_HEADER = JpegAscii.ORIG_ENCODING_HEADER;
  public static final String ASCII_ART_ENCODING = "asciiart";
  
  public AsciiJpeg(AdaptorWindow window, String parameter) {
    super(window, parameter);
  }
  
protected void handleContent(int contentLength, String contentType,
			     String contentEncoding)  
  throws AsyncEventException {
    if (contentType.startsWith("image/jpeg") && (contentEncoding != null) &&
	contentEncoding.equalsIgnoreCase(ASCII_ART_ENCODING)) {
      System.out.println(this + "AsciiArt -> JPEG");
      
      convertToJpeg(contentLength);
      
    } else {  // I don't know what it is, just forward it
      System.out.println(this + "Handling unknown data");
      writeHeaders();
      forwardContent(contentLength);
    }
    
    System.out.println(this + ": Done with content");
  }
  
  protected void convertToJpeg(int contentLength) throws AsyncEventException {
    contentLength = buildJpeg(window, currentPos, contentLength, 
			      (float) 0.3);
    
    header.deleteHeader("Content-Encoding");
    header.replaceHeaderValue("Content-Length", "" + contentLength);
     
    
    
    String origEncoding = header.getHeaderValue(ORIG_ENCODING_HEADER);
    
    if (origEncoding != null) {
      header.replaceHeaderValue("Content-Encoding", origEncoding);
      header.deleteHeader(ORIG_ENCODING_HEADER);
    }
    
    writeHeaders();
    
    forwardContent(contentLength);
  }
  
  static int buildJpeg(AdaptorWindow window, 
		       DataAccessPointer currentPos, 
		       int contentLength, float qualityLevel) {
    WindowInputStream is = new WindowInputStream(window, 
						 currentPos, contentLength);
    WindowOutputStream os = new WindowOutputStream(is);
    os.setSupressFlush(true);
    os.setAutoFlush(false);
    
    try {
      
      byte bytearrayorig[] = new byte[contentLength];
      int width, height, datastart;
      System.out.println("Content-Length: " + contentLength);
      DataInputStream dis = new DataInputStream(is);
      dis.readFully(bytearrayorig);
      byte bytearray[] = bytearrayorig;


	
      
      width = 1;
      datastart = 0;
      for(int i = 1,j = 0; (i < contentLength) && j!=1; i++) {
	if(bytearray[i] == ' ') {
	  width = Integer.parseInt(new String(bytearray,0,i));
	  width /= 2;
	  datastart = i+1;
	  System.out.println("Width: " + width); 
	  j=1; 
	}
	
      }

      height = (int) (contentLength/width);
      String tmp = new String(bytearray,datastart,width);
      
      Graphics2D g;
      
      //BufferedImage buffimg = new BufferedImage(width*2,(contentLength/width)*2,BufferedImage.TYPE_INT_ARGB);
      //g = buffimg.createGraphics();
      //set the font to a fixed size font
      Font f = new Font("courier",Font.BOLD,3);
      //g.setFont(f);
      //set the default color to white
      //g.setColor(Color.white);
      //get our local rendering cotnext
      //FontRenderContext context = g.getFontRenderContext();
      //get the size of a single line
      //      Rectangle2D bounds = f.getStringBounds(tmp,context);
      int lineheight = 5;
      int linewidth = 2*width;

      
      int newwidth = linewidth;
      int newheight = (contentLength/width) * lineheight; //# of lines * height


      BufferedImage outputimg = new BufferedImage(newwidth, newheight, BufferedImage.TYPE_BYTE_GRAY);
      //get the real graphics2d context
      g = outputimg.createGraphics(); 
      g.setFont(f);
      g.setColor(Color.white);
      System.out.println("writing out scan lines");
      
      int toskip = (int) Math.ceil(newheight/(height*2));
      
      for(int i = datastart, j=0; i < (contentLength-width); i+=width, j++) {

	g.drawString(new String(bytearray,i,width), 0, j * lineheight);
      }
      


      is.close();

/*
      System.out.println("creating scaled image");
      BufferedImage scaledimage = new BufferedImage(width*2, height*2, BufferedImage.TYPE_BYTE_GRAY);
      scaledimage.createGraphics().drawImage(outputimg.getScaledInstance(width*2,height*2,Image.SCALE_FAST),null,null);
      System.out.println("finished creating scaled image");
      outputimg = scaledimage;
*/
      System.out.println("getting jpeg params");      
      JPEGEncodeParam param = JPEGCodec.getDefaultJPEGEncodeParam(outputimg);
      param.setQuality(qualityLevel, false);
      System.out.println("creating encoder");
      JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(os,param);
      System.out.println("encoding Jpeg");
      encoder.encode(outputimg);
      System.out.println("written JPEG to stream: " + os.bytesWritten());

      

      
      os.close();
      
      return os.bytesWritten();
    } catch (IOException e) { e.printStackTrace();
      throw new RuntimeException("Case not handled.");
    } catch (ImageFormatException e) {
      e.printStackTrace();
	throw new RuntimeException("Case not handled.");
      }
  }
}
