/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package FMG.ConductorExt.adaptors.http;

import java.util.zip.*;
import java.io.*;

import FMG.Conductor.dataflow.*;
import FMG.Conductor.util.*;

import FMG.ConductorExt.adaptors.tools.*;

/**
 * This adaptor takes data that gzip encoded and uncompresses it.
 * <p>
 * @author Mark Yarvis (yarvis@fmg.cs.ucla.edu)
 * @version  $Revision: 1.4 $
 */

public class DecompressData extends HttpResponseAdaptor {

   private static final String ORIG_ENCODING_HEADER = 
                                     CompressText.ORIG_ENCODING_HEADER;

   public DecompressData(AdaptorWindow window, String parameter) {
      super(window, parameter);
   }

   protected void handleContent(int contentLength, String contentType,
                                                   String contentEncoding)  
                                              throws AsyncEventException {
      if ((contentEncoding != null) && 
          contentEncoding.equalsIgnoreCase("gzip")) {
         contentLength = decompressData(contentLength);
      }

      String origEncoding = header.getHeaderValue(ORIG_ENCODING_HEADER);

      if (origEncoding != null) {
         header.replaceHeaderValue("Content-Encoding", origEncoding);
         header.deleteHeader(ORIG_ENCODING_HEADER);
      }

      writeHeaders();
      forwardContent(contentLength);
   }

   int decompressData(int contentLength) throws AsyncEventException {
      WindowInputStream wis = new WindowInputStream(window, currentPos);
      WindowOutputStream wos = new WindowOutputStream(wis);
      wos.setAutoFlush(false);
      wos.setSupressFlush(true);
      try {
         GZIPInputStream is = new GZIPInputStream(wis);

         byte[] buf = new byte[10240];

         try {
            while(true) {
               int len = is.read(buf);
               if (len == -1) {
                  break;
               }
               wos.write(buf, 0, len);
            }
         } catch (EOFException e) {}
         wis.close();
         wos.close();
      } catch (IOException e) {
         throw new RuntimeException("This shouldn't happen");
      }

      header.replaceHeaderValue("Content-Length", "" + wos.bytesWritten());
      header.deleteHeader("Content-Encoding");

      return wos.bytesWritten();
   }
}
