/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package FMG.ConductorExt.adaptors.http;

import FMG.Conductor.dataflow.*;
import FMG.Conductor.util.*;
import FMG.ConductorExt.adaptors.ProtocolAdaptor;

/**
 * This class provides a useful base class for adaptors that operate
 * on HTTP streams.  It provides functions for dealing with the headers
 * and hooks for subclasses that understand the difference between the
 * upstream and downstream flow.
 * <p>
 * @author Mark Yarvis (yarvis@fmg.cs.ucla.edu)
 * @version $Revision: 1.16 $      
 */

public abstract class HttpAdaptor extends ProtocolAdaptor {

   protected Line firstLine;

   protected Header header = new Header();

   public HttpAdaptor(AdaptorWindow window, String parameter) {
      super(window, parameter);
   }

   protected abstract DataAccessPointer readFirstLine(
             DataAccessPointer endOfPrevious) throws AsyncEventException;

   // currentPos now points the the spot after the last \n or \r\n
   protected void readHeaders() throws AsyncEventException {
      DataAccessPointer endOfPrevious = new DataAccessPointer();
      currentPos.insertPointer(endOfPrevious);

      try {
         endOfPrevious = readFirstLine(endOfPrevious);

         header.readHeader(endOfPrevious);
      } catch (AsyncEventException e) {
         endOfPrevious.removeSelf();
         if (firstLine != null) {
            firstLine.getPostPointer().removeSelf();
         }
         throw e;
      }

      // success!  move currentPos
      currentPos.removeSelf();
      header.getPostPointer().insertPointer(currentPos);
   }

   // remember the values of all the headers so when we write them out,
   // we'll still know the values
   protected void rememberHeaders() {
      firstLine.cacheSelf();
      header.cacheSelf();
   }

   protected void writeHeaders() {
      // how much do we need to contract?
      int len = window.getPosition(header.getPostPointer());

      // pull up the firstLine pointers
      firstLine.getPrePointer().removeSelf();
      firstLine.getPostPointer().removeSelf();

      // pull up all heaer pointers
      header.removeSelf();

      // write out the header
      window.contract(len);
   }

}
