/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package FMG.ConductorExt.adaptors.http;

import java.util.Enumeration;
import java.io.*;

import com.sun.image.codec.jpeg.*;
import java.awt.image.BufferedImage;

import FMG.Conductor.dataflow.*;
import FMG.Conductor.util.*;

import FMG.ConductorExt.adaptors.tools.*;
import FMG.ConductorExt.adaptors.tools.image.*;

/**
 * This adaptor takes a jpeg image and renders it into an Ascii Art 
 * text image, using aalib.
 * <p>
 * @author Kevin Eustice (kfe@cs.ucla.edu)
 * @version $Revision: 1.9 $      
 */
public class JpegAscii extends HttpResponseAdaptor {
   public static final String ORIG_ENCODING_HEADER = "AA-Orig-Encoding";
   public static final String ASCII_ART_ENCODING_HEADER = "asciiart";



   public JpegAscii(AdaptorWindow window, String parameter) {
      super(window, parameter);
   }

   protected void handleContent(int contentLength, String contentType,
                                                   String contentEncoding)  
                                              throws AsyncEventException {
      if (contentType.startsWith("image/jpeg")) {
	System.out.println(this + "Handling JPEG");
	if(contentEncoding != null) {
	  header.addHeader(ORIG_ENCODING_HEADER,contentEncoding);
	  header.replaceHeaderValue("Content-Encoding",ASCII_ART_ENCODING_HEADER);
	}
	else {
	  header.addHeader("Content-Encoding",ASCII_ART_ENCODING_HEADER);

	}
	
	handleJpeg(contentLength);

      } else {  // I don't know what it is, just forward it
System.out.println(this + "Handling unknown data");
         writeHeaders();
         forwardContent(contentLength);
      }

System.out.println(this + ": Done with content");
   }

   protected void handleJpeg(int contentLength) throws AsyncEventException {
      contentLength = jpegToAscii(window, currentPos, contentLength);
      System.out.println("Content-Length: " + contentLength);
      header.replaceHeaderValue("Content-Length", "" + contentLength);
      writeHeaders();
      forwardContent(contentLength);
   }

   static int jpegToAscii(AdaptorWindow window, 
                                DataAccessPointer currentPos, 
                                int contentLength) {
      WindowInputStream is = new WindowInputStream(window, 
                                           currentPos, contentLength);
      WindowOutputStream os = new WindowOutputStream(is);
      os.setSupressFlush(true);
      os.setAutoFlush(false);

      try {
	System.out.println("Decoding JPEG image");
         JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(is);
         BufferedImage image = decoder.decodeAsBufferedImage();
	System.out.println("Grabbing RGB data");
	 int w = image.getWidth();
	 int h = image.getHeight();
	 int imgdata[] = new int[w*h+2];
	 int tmpdata[] = null;
	 char newimage[];
	 byte outputbuff[];
	 int rc = 0;
	 //	 image.getRGB(0,0,w,h,imgdata,0,w);
	 tmpdata = image.getData().getPixels(0,0,w,h,tmpdata);
	 System.out.println("Width: " + w + "\nHeight: " + h + "\nRaster size: " + tmpdata.length + "\nRaster components: " + tmpdata.length/(w*h));
	 rc = tmpdata.length/(w*h);
	 for(int i = 0; i < w*h;i++) {
	     imgdata[i] = 0;
	     for(int c = 0; c < rc; c++) 
		 imgdata[i] += tmpdata[rc*i+c];
	     imgdata[i] /= rc;
	     
	 }

	 imgdata[w*h] = w;
	 imgdata[w*h+1] = h;
	 System.out.println("Rendering ASCII");
	 newimage = RenderAsciiImage.RenderAscii(imgdata);
	 outputbuff = new byte[newimage.length];

	for(int i = 0;i<newimage.length;i++) {
	  outputbuff[i] = (byte) (newimage[i] & 0xff);
	} 
	
	
	os.write(((new Integer(w)).toString()).getBytes());
	os.write(' ');
	is.close();	 
	os.write(outputbuff);
	os.close();


         return os.bytesWritten();
      } catch (IOException e) {
         throw new RuntimeException("Case not handled.");
      } catch (ImageFormatException e) {
         e.printStackTrace();
         throw new RuntimeException("Case not handled.");
      }
   }
}
