/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package FMG.ConductorExt.adaptors.http;

import FMG.Conductor.dataflow.*;
import FMG.Conductor.util.Debug;

/**
 * This adaptor looks for HTTP requests that have been mangled by the 
 * LowsrcEnabler and unmangles them.  It also notifies a LowsrcEnabler
 * running on the same node (for this stream) that the results of this
 * request should be adapted.
 * <p>
 * Typically this adaptor is run on the same node as a LowsrcEnabler.
 * <p>
 * @author Mark Yarvis (yarvis@fmg.cs.ucla.edu)
 * @version $Revision: 1.6 $      
 */

public class LowsrcDetector extends HttpRequestAdaptor {

   static final String LOW_SRC_SUFFIX = LowsrcEnabler.LOW_SRC_SUFFIX;

   static final String COMM_TAG = "LowsrcImage";
   static final String TRUE = "true";
   static final String FALSE = "false";

   protected IAC iac;

   public LowsrcDetector(AdaptorWindow window, String parameter) {
      super(window, parameter);
      iac = getStreamIAC();
   }

   public void handleRequest(String url) {
      if (Debug.on) {
         Debug.println(Debug.ADAPTOR, Debug.INFO, 
                       this + "Got URL request: " + url);
      }

      if (url.endsWith(LOW_SRC_SUFFIX)) {
         url = url.substring(0, url.length() - LOW_SRC_SUFFIX.length());
         if (Debug.on) {
            Debug.println(Debug.ADAPTOR, Debug.INFO, 
                          this + "Switched URL to: " + url);
         }

         requestLine.replaceTarget(url);

         iac.store(COMM_TAG, TRUE, true);
      } else {
         iac.store(COMM_TAG, FALSE, true);
      }

      writeHeaders();
   }
}
