/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package FMG.ConductorExt.adaptors.imagedb;

import FMG.Conductor.dataflow.*;
import FMG.Conductor.util.Measure;

public abstract class ImageDBAdaptor extends Adaptor {

   public ImageDBAdaptor(AdaptorWindow window, String parameter) {
      super(window, parameter);
   }

   protected int min(int i, int j) {
      return (i>j? j : i);
   }

   protected int min(int i, int j, int k) {
      return (i<j ? (i<k ? i : k) : (j<k ? j : k));
   }

   protected void getNextLine(BlockModPointer pointer) 
                           throws InterruptedException, AsyncEventException {
      if (pointer.distanceTo((byte) '\n') == -1) {
         window.expand((byte) '\n');
      }
   }

   protected int readInt(BlockModPointer pointer) {
      byte b;
      int result = 0;
      boolean isNeg = false;

      int i = 0;
      b = pointer.getByte(i++);
      if (b == '-') {
         isNeg = true;
         b = pointer.getByte(i++);
      }

      while ((b != (byte) ' ') && (b != (byte) '\n')) {
         result = (result * 10) + b - (byte) '0';
         b = pointer.getByte(i++);
      }

      return (result * (isNeg ? -1 : 1));
   }

   protected long futureTime(int msec) {
      return Measure.getTime() + msec;
   }

   protected boolean pastTime(long time) {
      return (Measure.getTime() > time);
   }  

   protected void waitForTime(long time) throws InterruptedException {
      long current = Measure.getTime();
      if (current > time) {
         return;
      }

      Thread.currentThread().sleep(time - current);
   }

}
