/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package FMG.ConductorExt.adaptors.imagedb;

import FMG.Conductor.util.*;
import FMG.Conductor.comm.*;
import FMG.Conductor.dataflow.*;

/*
 * NOTE: this adaptor probably still violates the rules of segmentation.
 *       we need more consistent use of BlockModPointer
 */

public class PowerSaver3 extends ImageDBAdaptor {

   static int instances = 0;
   static char echochars[] = { '.', ',', ';', ':', '|'};

   char e;
   int buflen = 1000000;
   byte[] inbuf = new byte[buflen];
   byte[] outbuf = new byte[buflen];
   IAC iac;

   public PowerSaver3(AdaptorWindow window, String parameter) {
      super(window, parameter);
      if (Debug.on) {
         Debug.println(Debug.ADAPTOR, Debug.INFO, 
                       "New PowerSaver3 instantiated");

         if (Debug.getLevel(Debug.ADAPTOR) > Debug.DETAIL) {
            e = echochars[(instances++) % echochars.length];
         }
      }

      iac = getStreamIAC();
   }

   public void skipBlock(BlockModPointer pointer, int len, long sendTime)
                         throws InterruptedException, AsyncEventException {
      while (len > 0) {
         if (window.countRemaining(pointer) == 0) {
            window.expand();
         }
         int toGo = min(len, window.countRemaining(pointer));
         pointer.advance(toGo);
         if (pastTime(sendTime)) {
            window.contract(pointer);
         }
         len -= toGo;
      }
   }

   public void handleFlow() {

      if (Debug.on) {
         Debug.print(Debug.ADAPTOR, Debug.INFO, "(");
      }

      BlockModPointer pointer = new BlockModPointer();
      window.insertPointer(pointer);

      try {
         while (true) {
            try {
               getNextLine(pointer);               /* get query/result id */
               long sendTime = ((Long) iac.lookup("SendTime")).longValue();
               iac.remove("SendTime");
               pointer.movePastByte((byte) '\n');
               if (pastTime(sendTime)) {
                  window.contract(pointer);           /* push it out */
               }

               getNextLine(pointer);               /* get # of samples */
               int numSamples = readInt(pointer);
               pointer.movePastByte((byte) '\n');
               if (pastTime(sendTime)) {
                  window.contract(pointer);           /* push it out */
               }

// Useful for debugging on ethernet
//System.out.println("Pausing ... slow network");
//Thread.currentThread().sleep(10000);
//System.out.println("Continuing");

               for (int i=0; i<numSamples; i++) {
                  getNextLine(pointer);             /* get image dimensions */
                  int size = 3;
                  size *= readInt(pointer);
                  pointer.movePastByte((byte) ' ');
                  size *= readInt(pointer);
                  pointer.movePastByte((byte) '\n');
                  if (pastTime(sendTime)) {
                     window.contract(pointer);         /* push them out */
                  }

                  if (i == numSamples - 1) {
                     pointer.blockModStart();
                  }
                  skipBlock(pointer, size, sendTime);
               }

               // append the ammount our sleep time was off for this request
               long finishTime = Measure.getTime();
               Integer difference = new Integer((int) (finishTime - sendTime));
               byte[] diffBytes = difference.toString().getBytes();
               pointer.insertBytes(diffBytes, 0, diffBytes.length);
               diffBytes[0]=(byte)'\n';
               pointer.insertBytes(diffBytes, 0, 1);
               pointer.blockModEnd();

               // send everything once we're sure it's time
               waitForTime(sendTime);
               window.contract(pointer);
            } catch (InputClosedException e) {
               pointer.removeSelf();
               window.contract();
               if (Debug.on) {
                  Debug.print(Debug.ADAPTOR, Debug.INFO, ")");
               }
               window.closeOutput();
               return;
            } catch (OutputClosedException e) {
               pointer.removeSelf();
               window.closeInput();
               return;
            } catch (AsyncEventException e) {
               throw new RuntimeException("Not implemented");
            }
         }
      } catch (InterruptedException ignore) {
         throw new RuntimeException("Case not handled");
      }
   }
}
