/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package FMG.ConductorExt.adaptors.pop3;
   
import FMG.Conductor.dataflow.*;
import FMG.Conductor.util.Debug;

import FMG.ConductorExt.adaptors.tools.*;

import java.io.*;
import java.util.zip.*;

/*
 * This adaptor is used to decompress mail messages downloaded using the POP3 
 * protocol.  It is assumed that they were previously compressed by a 
 * MessageCompressor adaptor.  The standard Java decompression classes are 
 * used.
 * <p>
 * This adaptor should be used in conjunction with the MessageCompressor, 
 * which should be upstream.
 * <p>
 * This adaptor assumes that some sort of Pop3RequestAdaptor is present
 * on the same node to decipher the request, so we know what type of response 
 * to expect.
 * <p>
 * @author Mark Yarvis (yarvis@fmg.cs.ucla.edu)
 * @version $Revision: 1.8 $      
 */
public class MessageDecompressor extends Pop3ResponseAdaptor {

   static final String COMPRESSED_RESULT = 
                                    MessageCompressor.COMPRESSED_RESULT;

   public MessageDecompressor(AdaptorWindow window, String parameter) {
      super(window, parameter);
   }

   protected void decompressMessage() throws AsyncEventException {

      WindowInputStream wis = new WindowInputStream(window, currentPos);
      WindowOutputStream wos = new WindowOutputStream(wis);
      wos.setAutoFlush(true);
      try {
//         GZIPInputStream is = new GZIPInputStream(wis);
         InflaterInputStream is = new InflaterInputStream(wis);

         byte[] buf = new byte[10240];

         try {
            while(true) {
               int len = is.read(buf);
               if (len == -1) {
                  break;
               }
               wos.write(buf, 0, len);
            }
         } catch (EOFException e) {}
         wis.close();
         wos.close();
      } catch (IOException e) {
         throw new RuntimeException("This shouldn't happen");
      }

   }

   protected boolean handleResponse(ReadOnlyRequest request, 
                                    boolean isInitialHandshake)
                                                 throws AsyncEventException {
      if (! isInitialHandshake && 
          response.getResult().equals(COMPRESSED_RESULT)) {
         response.setSuccessfulResult();
         writeResponse();

         decompressMessage();
         return true;
      }

      // if we didn't handle it above
      return super.handleResponse(request, isInitialHandshake);
   }

}
