/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package FMG.ConductorExt.adaptors.pop3;

import java.io.*;

import FMG.Conductor.dataflow.*;
import FMG.Conductor.util.Debug;

import FMG.ConductorExt.adaptors.tools.*;

/**
 * This adaptor prioritizes a POP3 stream based on the priority reported by
 * the PriorityDetector adaptor.
 * <p>
 * This adaptor should be deployed downstream of a PriorityDetector adaptor.
 * <p>
 * This adaptor assumes that some sort of Pop3RequestAdaptor is present
 * on the same node to decipher the request, so we know what type of response 
 * to expect.
 * <p>
 * @author Mark Yarvis (yarvis@fmg.cs.ucla.edu)
 * @version $Revision: 1.8 $      
 */
public class Prioritizer extends Pop3ResponseAdaptor {

   public static final int DEFAULT_PRIORITY = 0;

   public Prioritizer(AdaptorWindow window, String parameter) {
      super(window, parameter);
   }

   protected boolean handleResponse(ReadOnlyRequest request,
                                     boolean isInitialHandshake)
                                                 throws AsyncEventException {
      // if a PASS request was issued, check for a priority level
      if (! isInitialHandshake && response.successfulResult()) {
         if (request.getCommand().equalsIgnoreCase("PASS")) {
            int numArgs = response.getNumArgs();

System.out.println("password response: " + response);
            if (response.getArg(numArgs - 2).equals("priority")) {
               int priority = DEFAULT_PRIORITY;
               try {
                  priority = Integer.parseInt(response.getArg(numArgs - 1));
               } catch (NumberFormatException ignore) {}

               response.replaceArg(numArgs - 1, "");
               response.replaceArg(numArgs - 2, "");

System.out.println("Setting the priority to " + priority);

               // set up the priority and handle the rest of the stream 
               writeResponse();
               window.requestDownstreamBuffering(AdaptorWindow.MIN_BUFFERING);
               PriorityWriter priorityWriter = 
                      new PriorityWriter(window, priority, getNodeIAC());
               while(true) {
                  priorityWriter.writeData(Integer.MAX_VALUE);
               }
            }
         }
      }

      return super.handleResponse(request, isInitialHandshake);
   }

}
