/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package FMG.ConductorExt.adaptors.test;

import FMG.Conductor.util.*;
import FMG.Conductor.comm.*;
import FMG.Conductor.dataflow.*;

/**
 * This adaptor does nothing, but it is able to be paired with other adaptors.
 * <p>
 * @author Mark Yarvis (yarvis@fmg.cs.ucla.edu)
 * @version $Revision: 1.2 $
 */

public class Paired extends Adaptor {

   public Paired(AdaptorWindow window, String parameter) {
      super(window, parameter);

      if (Debug.on) {
         Debug.println(Debug.ADAPTOR, Debug.INFO, 
                       this + " instantiated");
      }
   }

   public void handleFlow() {
      if (Debug.on) {
         Debug.print(Debug.ADAPTOR, Debug.INFO, "(");
      }

      while (true) {
         try {
            int len = window.expand();
            if (Debug.on) {
               Debug.println(Debug.ADAPTOR, Debug.DETAIL,
                             this + ": moving " + len + " bytes");
            }

            window.contract(len);
         } catch (InputClosedException e) {
            window.contract();
            window.closeOutput();
            if (Debug.on) {
               Debug.print(Debug.ADAPTOR, Debug.INFO, ")");
            }
            return;
         } catch (OutputClosedException e) {
            if (Debug.on) {
               Debug.print(Debug.ADAPTOR, Debug.INFO, ")");
            }
            window.closeOutput();
            return;
         } catch (PlanChangedException e) {
             window.contract();
             window.advanceSwitchPlan();
         } catch (RollbackOccurredException e) {
            Debug.println(this + " Rolling back.");
            window.contract();
         } catch (AsyncEventException e) {
            throw new RuntimeException("Unexpected exception: " + e);
         }
      }
   }

   public static CompositionType compositionType(String parameters) {
      if (parameters == null) {
         return CompositionType.NONE;
      } else if (parameters.equals("do")) {
         return CompositionType.PUSH;
      } else if (parameters.equals("undo")) {
         return CompositionType.POP;
      } else {
         return CompositionType.NONE;
      }
   }
}
