/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package FMG.ConductorExt.planners.template;

import java.util.*;
import FMG.ConductorExt.planners.template.numbers.*;

class AttrCompList {

   protected class AttrComp {
      protected String name;
      protected ValueComp valueComp;

      protected AttrComp(String text) throws FieldFormatException {
         int funcPos;
         int funcLen;
         ValueComp.CompFunc func;

         if ((funcPos = text.indexOf("==")) != -1) {
            funcLen = 2;
            func = ValueComp.CMP_EQ;
         } else if ((funcPos = text.indexOf("!=")) != -1) {
            funcLen = 2;
            func = ValueComp.CMP_NEQ;
         } else if ((funcPos = text.indexOf(">=")) != -1) {
            funcLen = 2;
            func = ValueComp.CMP_GE;
         } else if ((funcPos = text.indexOf(">")) != -1) {
            funcLen = 1;
            func = ValueComp.CMP_GT;
         } else if ((funcPos = text.indexOf("<=")) != -1) {
            funcLen = 2;
            func = ValueComp.CMP_LE;
         } else if ((funcPos = text.indexOf("<")) != -1) {
            funcLen = 1;
            func = ValueComp.CMP_LT;
         } else {
            throw new FieldFormatException("Bad or missing modifier function");
         }

         name = text.substring(0, funcPos);

         try {
            int dotPos;
            if ((dotPos = name.indexOf('.')) != -1) {
               String funcName = name.substring(dotPos+1, name.length());
               name = name.substring(0, dotPos);

               Class aggregationClass;

               if (funcName.equals("ave")) {
                  aggregationClass = AverageValue.class;
               } else if (funcName.equals("sum")) {
                  aggregationClass = SumValue.class;
               } else if (funcName.equals("min")) {
                  aggregationClass = MinValue.class;
               } else if (funcName.equals("max")) {
                  aggregationClass = MaxValue.class;
               } else {
                  throw new FieldFormatException("Unknown aggregation " + 
                                                 "function");
               }

               valueComp = new ValueComp(
                                   new Value(text.substring(funcPos+funcLen, 
                                                            text.length())), 
                                   func, aggregationClass);

            } else {
               valueComp = new ValueComp(
                                  new Value(text.substring(funcPos+funcLen, 
                                                           text.length())), 
                                  func);
            }
         } catch (NumberFormatException e) {
            throw new FieldFormatException("Bad comparison value");
         }
      }

      public String toString() {
         return (name + valueComp);
      }
   }

   Hashtable comps = new Hashtable();

   AttrCompList(Hashtable source, String baseName) 
                                            throws FieldFormatException {
      for (int i=0; true; i++) {
         String val = (String) source.get(baseName + i);
         if (val == null) {
            break;
         }
         try {
            AttrComp newAttrComp = new AttrComp(val);
            comps.put(newAttrComp.name, newAttrComp);
         } catch (FieldFormatException e) {
            throw new FieldFormatException("In modifier '" + baseName + i + 
                                           "': " + e);
         }
      }
   }

   boolean verify(AttributeList[] attrs, AttrModList[] mods) {
      if (attrs.length != mods.length) {
         throw new RuntimeException("This shouldn't happen.");
      }

      int numAttrs = attrs.length;

      Value[] values = new Value[numAttrs];
      ValueMod[] valueMods = new ValueMod[numAttrs];

      Enumeration names = comps.keys();

      while (names.hasMoreElements()) {
         String n = (String) names.nextElement();

         for (int i=0; i<numAttrs; i++) {
            values[i]=attrs[i].getValue(n);
            valueMods[i] = (mods[i]==null ? null : mods[i].getMod(n));
         }

         if (! ((AttrComp) comps.get(n)).valueComp.test(values, valueMods)) {
            return false;
         }
      }

      return true;
   }

   boolean verify(AttributeList attrs) {
      Enumeration names = comps.keys();

      while (names.hasMoreElements()) {
         String n = (String) names.nextElement();

         if (! ((AttrComp) comps.get(n)).valueComp.test(attrs.getValue(n))) {
            return false;
         }
      }
      return true;
   }

   public String toString() {
      StringBuffer sb = new StringBuffer();
      Enumeration e = comps.elements();
      while (e.hasMoreElements()) {
         sb.append(e.nextElement().toString());
         if (e.hasMoreElements()) {
            sb.append(",");
         }
      }
      return sb.toString();
   }
}
