/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package FMG.ConductorExt.planners.template;

import java.util.*;

import FMG.ConductorExt.planners.template.numbers.*;

class AttrModList {
   Hashtable mods = new Hashtable();

   protected class AttrMod {
      protected String name;
      protected ValueMod mod;

      protected AttrMod(String text) throws FieldFormatException {
         int equalsPos = text.indexOf('=');

         if (equalsPos == -1) {
            throw new FieldFormatException("Badly formed modifier");
         }

         try {
            Value value = new Value(text.substring(equalsPos+1, 
                                                   text.length()));

            switch (text.charAt(equalsPos-1)) {
               case '+':
                  name = text.substring(0, equalsPos-1);
                  mod = new ValueMod(value, ValueMod.FUNC_ADD);
                  break;
               case '-':
                  name = text.substring(0, equalsPos-1);
                  mod = new ValueMod(value, ValueMod.FUNC_SUB);
                  break;
               case '*':
                  name = text.substring(0, equalsPos-1);
                  mod = new ValueMod(value, ValueMod.FUNC_MULT);
                  break;
               case '/':
                  name = text.substring(0, equalsPos-1);
                  mod = new ValueMod(value, ValueMod.FUNC_DIV);
                  break;
               default:
                  name = text.substring(0, equalsPos);
                  mod = new ValueMod(value, ValueMod.FUNC_ASSIGN);
            }
         } catch (NumberFormatException e) {
            throw new FieldFormatException("Bad modifier value");
         }

      }

      public String toString() {
         return name + mod;
      }
   }

   AttrModList(Hashtable source, String baseName) throws FieldFormatException {
      for (int i=0; true; i++) {
         try {
            String val = (String) source.get(baseName + i);
            if (val == null) {
               break;
            }
            AttrMod newAttrMod = new AttrMod(val);
            mods.put(newAttrMod.name, newAttrMod);
         } catch (FieldFormatException e) {
            throw new FieldFormatException("Field '" + baseName + i + "':" + e);
         }
      }
   }

   ValueMod getMod(String target) {
      AttrMod a = (AttrMod) mods.get(target);
      if (a == null) {
         return null;
      } else {
         return a.mod;
      }
   }

   public String toString() {
      StringBuffer sb = new StringBuffer();
      Enumeration e = mods.elements();
      while (e.hasMoreElements()) {
         sb.append(e.nextElement().toString());
         if (e.hasMoreElements()) {
            sb.append(",");
         }
      }
      return sb.toString();
   }
}
