/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package FMG.ConductorExt.planners.template;

import java.util.*;

import FMG.Conductor.util.Debug;

class Requirements {

   ItemList preferredDataMods;
   ItemList badDataMods;

   AttrCompList netRequirements;

   Requirements(Hashtable infoTable) {
      preferredDataMods = new ItemList(infoTable, 
                                       "UserPrefs.DataMods.Preferred.");
      badDataMods = new ItemList(infoTable, "UserPrefs.DataMods.Bad.");

      try {
         netRequirements = new AttrCompList(infoTable, 
                                         "UserPrefs.LinkRequirement.");
      } catch (FieldFormatException e) { 
         if (Debug.on) {
            Debug.println(Debug.WARNING, 
                          "Bad network requriement in user prefs: " + e);
         }
      }
   }

   boolean hasAcceptableDataMods(Template t) {
      String[] mods = t.getDataMods().getItems();

      for (int i = 0; i < mods.length; i++) {
         if (badDataMods.contains(mods[i])) {
            return false;
         }
      }

      return true;
   }

   int getDataModRanking(String mod) {
      int rank = preferredDataMods.getIndex(mod);
      if (rank < 0) {
         return preferredDataMods.size() + 1;
      } else {
         return rank + 1;
      }
   }

   AttrCompList getNetRequirements() {
      return netRequirements;
   }

   public String toString() {
      StringBuffer sb = new StringBuffer("Requirements[");
      sb.append("preferredMods:");
      sb.append(preferredDataMods.toString());
      sb.append("badMods:");
      sb.append(badDataMods.toString());
      sb.append("netReqs:");
      sb.append(netRequirements.toString());
      sb.append("]");
      return sb.toString();
   }

}
