/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package FMG.ConductorExt.planners.template.numbers;

public class Value {
   protected long longValue;
   protected double doubleValue;

   static final int TYPE_UNASSIGNED=0;
   static final int TYPE_BOOL=1;
   static final int TYPE_LONG=2;
   static final int TYPE_DOUBLE=3;

   protected int dataType = TYPE_UNASSIGNED;

   public Value (String text) {
      long multiplier = 1;

      switch (text.charAt(text.length()-1)) {
         case 'k':
         case 'K': { multiplier = 1000; break; }
         case 'm':
         case 'M': { multiplier = 1000000; break; }
         case 'g':
         case 'G': { multiplier = 1000000000; break; }
      }
      if (multiplier > 1) {
         text = text.substring(0, text.length()-1);
      }

      if (text.equals("true")) {               // boolean true specified
         longValue = 1;
         doubleValue = 1;
         dataType = TYPE_BOOL;
      } else if (text.equals("false")) {       // boolean false specified
         longValue = 0;
         doubleValue = 0;
         dataType = TYPE_BOOL;
      } else if (text.indexOf('.')!=-1) {     // double value specified
         doubleValue = Double.valueOf(text).doubleValue() * multiplier;
         longValue = (long) doubleValue;
         dataType = TYPE_DOUBLE;
      } else {                                // long value specified
         longValue = Long.parseLong(text) * multiplier;
         doubleValue = (double) longValue;
         dataType = TYPE_LONG;
      }
   }

   public Value(boolean newVal) {
      longValue = (newVal ? 1 : 0);
      doubleValue = (newVal ? 1 : 0);
      dataType = TYPE_BOOL;
   }

   public Value(long newVal) {
      longValue = newVal;
      doubleValue = (double) newVal;
      dataType = TYPE_LONG;
   }

   public Value(double newVal) {
      longValue = (long) newVal;
      doubleValue = newVal;
      dataType = TYPE_DOUBLE;
   }

   double doubleValue() {
      return doubleValue;
   }

   long longValue() {
      return longValue;
   }

   boolean booleanValue() {
      return (longValue != 0);
   }

   int dataType() {
      return dataType;
   }

   public String toString() {
      switch (dataType) {
         case TYPE_UNASSIGNED:
            return("unassigned");
         case TYPE_BOOL:
            return ((longValue != 0) ? "true" : "false");
         case TYPE_LONG:
            return ("" + longValue);
         case TYPE_DOUBLE:
            return ("" + doubleValue);
         default:
            throw new RuntimeException("This should never happen");
      }
   }
}
