/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package FMG.ConductorExt.planners.template.numbers;

public class ValueMod {
   Value value;
   ModFunc func;

   private static ValueMod dummy = new ValueMod();

   public abstract class ModFunc {
      int resultType(Value modVal, Value target) {
         int type1 = modVal.dataType();
         int type2 = target.dataType();

         // return larger type
         return (type1>type2 ? type1 : type2);
      }

      abstract boolean booleanResult(Value modVal, Value target);
      abstract long longResult(Value modVal, Value target);
      abstract double doubleResult(Value modVal, Value target);
      abstract public String toString();
   }

   public static ModFunc FUNC_ASSIGN = dummy. new ModFunc() {
      boolean booleanResult(Value modVal, Value target) {
         return modVal.booleanValue();
      }

      long longResult(Value modVal, Value target) {
         return modVal.longValue();
      }

      double doubleResult(Value modVal, Value target) {
         return modVal.doubleValue();
      }

      public String toString() {
         return "=";
      }
   };

   public static ModFunc FUNC_ADD = dummy. new ModFunc() {
      boolean booleanResult(Value modVal, Value target) {
         return modVal.booleanValue() | target.booleanValue();
      }

      long longResult(Value modVal, Value target) {
         return modVal.longValue() + target.longValue();
      }

      double doubleResult(Value modVal, Value target) {
         return modVal.doubleValue() + target.doubleValue();
      }

      public String toString() {
         return "+=";
      }
   };

   public static ModFunc FUNC_SUB = dummy. new ModFunc() {
      boolean booleanResult(Value modVal, Value target) {
         return modVal.booleanValue() | target.booleanValue();
      }

      long longResult(Value modVal, Value target) {
         return target.longValue() - modVal.longValue();
      }

      double doubleResult(Value modVal, Value target) {
         return target.doubleValue() - modVal.doubleValue();
      }

      public String toString() {
         return "-=";
      }
   };

   public static ModFunc FUNC_MULT = dummy. new ModFunc() {
      boolean booleanResult(Value modVal, Value target) {
         return modVal.booleanValue() & target.booleanValue();
      }

      long longResult(Value modVal, Value target) {
         return target.longValue() * modVal.longValue();
      }

      double doubleResult(Value modVal, Value target) {
         return target.doubleValue() * modVal.doubleValue();
      }

      public String toString() {
         return "*=";
      }
   };

   public static ModFunc FUNC_DIV = dummy. new ModFunc() {
      boolean booleanResult(Value modVal, Value target) {
         return modVal.booleanValue() & target.booleanValue();
      }

      long longResult(Value modVal, Value target) {
         return target.longValue() / modVal.longValue();
      }

      double doubleResult(Value modVal, Value target) {
         return target.doubleValue() / modVal.doubleValue();
      }

      public String toString() {
         return "/=";
      }
   };

   private ValueMod() {
   }

   public ValueMod(Value value, ModFunc func) {
      this.value = value;
      this.func = func;
   }

   public int resultType(Value target) {
      return func.resultType(value, target);
   }

   public boolean booleanResult(Value target) {
      return func.booleanResult(value, target);
   }

   public long longResult(Value target) {
      return func.longResult(value, target);
   }

   public double doubleResult(Value target) {
      return func.doubleResult(value, target);
   }

   public String toString() {
      return func.toString() + value.toString();
   }
}
