/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package FMG.ConductorExt.security.adaptors;

import java.security.*;
import xjava.security.Cipher;

import FMG.Conductor.dataflow.*;
import FMG.Conductor.util.*;

import FMG.ConductorExt.adaptors.pop3.Pop3RequestAdaptor;

import cryptix.provider.key.RawKey;

/**
 * <p>
 * @author Mark Yarvis (yarvis@fmg.cs.ucla.edu)
 * @version $Revision: 1.7 $
 */
public class Pop3PasswordEncryptor extends Pop3RequestProtector {

   protected int commandsSinceAuth = -1;

   public Pop3PasswordEncryptor(AdaptorWindow window, String parameter) {
      super(window, parameter);

      try {
          cipher = Cipher.getInstance(ENCRYPTION_ALG + "/" + DEFAULT_MODE);
          cipher.initEncrypt(getKey());
      } catch ( NoSuchAlgorithmException e ) {
          throw new RuntimeException("No such algorithm as "+
                                     ENCRYPTION_ALG + "/" + DEFAULT_MODE);
      } catch ( KeyException e ) {
          throw new RuntimeException("KeyException in cipher initialization");
      }
   }

   public boolean handleRequest(Request request) {
      if (commandsSinceAuth != -1) {
         commandsSinceAuth++;
      }

      if (request.getCommand().equalsIgnoreCase("PASS")) {
         // encrypt the password
         byte[] password = request.getArg(0).getBytes();
         byte[] protectedPass = hexEncode(cipher.crypt(password));
         request.replaceArg(0, protectedPass);

         // don't call handleRequest so we don't overwrite unencrypted info

         // no more to do in this stream
         return false;
      } else if (request.getCommand().equalsIgnoreCase("AUTH") && 
                 (request.getNumArgs() != 0)) {
         if (request.getArg(0).equalsIgnoreCase("LOGIN")) {
            commandsSinceAuth = 0;
         } else {
            Debug.println(Debug.ADAPTOR, Debug.WARNING,
                          this + "Warning: unknown auth type: " +
                          request.getArg(0));
         }
      } else if (commandsSinceAuth == 2) {
         // encrypt the password
         byte[] password = request.getCommand().getBytes();
         byte[] protectedPass = hexEncode(cipher.crypt(password));
         request.replaceCommand(protectedPass);

         // don't call handleRequest so we don't overwrite unencrypted info

         // no more to do in this stream
         return false;
      }

      return super.handleRequest(request);
   }

   public static String getKeyLabel(String parameters) {
      return Pop3RequestProtector.getKeyLabel(parameters);
   }
   
   public static byte[] generateKey(String parameters) {
      return Pop3RequestProtector.generateKey(parameters);
   }

}
