/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package FMG.ConductorExt.security.adaptors;

import java.security.*;
import xjava.security.Cipher;
import xjava.security.KeyGenerator;

import FMG.Conductor.dataflow.*;
import FMG.Conductor.util.*;

import FMG.ConductorExt.adaptors.pop3.Pop3RequestAdaptor;

import cryptix.provider.key.RawKey;

/**
 * <p>
 * @author Mark Yarvis (yarvis@fmg.cs.ucla.edu)
 * @version $Revision: 1.6 $
 */
public abstract class Pop3RequestProtector extends Pop3RequestAdaptor {

   protected static final String ENCRYPTION_ALG = "DES";
   private static final String DEFAULT_KEYLABEL = ENCRYPTION_ALG;
   protected static final String DEFAULT_MODE = "CFB";

   protected Cipher cipher;

   public Pop3RequestProtector(AdaptorWindow window, String parameter) {
      super(window, parameter);
   }

   protected Key getKey() {
      byte[] keyBytes = null;

      while (keyBytes == null) {
         try {
            keyBytes = (byte[]) getStreamIAC().waitFor(getKeyLabel(null));
         } catch (InterruptedException ignore) { }
      }

      return new RawKey(getKeyLabel(null), keyBytes);
   }

   public static String getKeyLabel(String parameters) {
      return DEFAULT_KEYLABEL;
   }
   
   public static byte[] generateKey(String parameters) {
      KeyGenerator keygen= null;
     
      try {
         SecureRandom rand = new SecureRandom();
         keygen = KeyGenerator.getInstance(getKeyLabel(parameters));
         keygen.initialize(rand);
      } catch ( Exception e ) {
         throw new RuntimeException("Key generator initialization failed:" + e);
      }
      
      return keygen.generateKey().getEncoded();
   }

   private static final byte hexByte(int n) {
      return (byte) (n>9 ? 'A' + n - 10 : '0' + n);
   }

   protected static byte[] hexEncode(byte in[]) {
      byte[] out = new byte[in.length * 2];
      for (int i = 0; i<in.length; i++) {
         int j = i*2;
         // encode high part of byte
         out[j] = hexByte((in[i] & 0xf0) >> 4);
         // encode low part of byte
         out[j+1] = hexByte(in[i] & 0xf);
      }
      return out;
   }

   private static final int hexVal(byte b) {
      if ((b >= '0') && (b <= '9')) {
         return (b - '0');
      } else if ((b >= 'A') && (b <= 'F')) {
         return (b - 'A' + 10);
      } else if ((b >= 'a') && (b <= 'f')) {
         return (b - 'a' + 10);
      } else {
         throw new RuntimeException("Bad argument");
      }
   }
 
   protected static byte[] hexDecode(byte in[]) {
      if (Debug.on) {
         if ((in.length % 2) != 0) {
            throw new RuntimeException("Bad argument");
         }
      }

      byte[] out = new byte[in.length / 2];

      for (int i=0; i<out.length; i++) {
         int j=i*2;
         out[i] = (byte) ((hexVal(in[j])<<4) + hexVal(in[j+1]));
      }

      return out;
   }

   // for testing only
   public static void main(String[] args) {
       System.out.println("Original => '" + args[0] + "'");
       byte[] encodedBytes = hexEncode(args[0].getBytes());

       System.out.print("Encoded => ");
       for (int i=0; i<encodedBytes.length; i++) {
          System.out.print((char)encodedBytes[i]);
       }
       System.out.println();

       byte[] decodedBytes = hexDecode(encodedBytes);

       System.out.print("Decoded => ");
       for (int i=0; i<decodedBytes.length; i++) {
          System.out.print((char)decodedBytes[i]);
       }
       System.out.println();
   }

}
