/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package FMG.ConductorExt.security.messages;

import  java.io.*;
import  java.net.*;
import  java.util.*;

import  FMG.Conductor.comm.*;
import  FMG.Conductor.util.*;

import  FMG.ConductorExt.security.*;

/**
 * @author Jun Li (lijun@fmg.cs.ucla.edu)
 * @version $Revision: 1.3 $
 */ 

public abstract class Authenticator extends SecurityMessage {

    /** The certificates useful for forming or updating the Authenticator */
    protected static Hashtable  localCerts = new Hashtable();

    /** The direction that this Authenticator flows */
    protected Direction         direction;

    
    /**
     * Read the data elements of this object from the input stream.  Presumably
     * this information was generated by <code>sendSelf()</code>.
     * @param is The input stream to read the message's data from.
     * @return No return value.
     * @exception java.io.IOException if an error occurs while reading from 
     *                                the stream
     */
    public void receiveSelf(DataInputStream is) throws IOException {
      super.receiveSelf(is);
      direction   = Direction.readDir(is);
    }

    /**
     * Write the data elements of this object to an output stream.  Presumably
     * the information will be read by <code>receiveSelf()</code>.
     * @param os The output stream to write the message's data to.
     * @return No return value.
     * @exception java.io.IOException if an error occurs while writing to the
     *                                stream.
     */
    public void sendSelf(DataOutputStream os) throws IOException {
       super.sendSelf(os);
       direction.writeSelf(os);
    }

    /**
     * Determine the direction of motion for this object.
     * @return returns <code>Direction.up</code> if towards the server;
     *                 <code>Direction.down</code> if towards the client.
     */
    public Direction flowDir() {
        return direction;
    }
    

    /*
     * private functions
     */

    /**
     * Initializes an Authenticator message.
     * <p>
     * It assigns the connection ID, the flow direction, the NodeID of the
     * node and its local certificate server. In particular, a certificate
     * for this node by the local certificate server will be retrieved, and
     * added to the list keeping the certificates for the node.
     *
     * @param connID  The id of the connection for which we are planning.
     * @param dir     The direction that this Authenticator will flow
     * @return no return value.
     */
    protected void init(ConnID connID, 
			Direction dir) {
        initMessage(connID);
        this.direction   = dir;
    }

                                                                   
}
