/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package FMG.ConductorExt.security.util;

import java.io.*;

/**
 * This BytesReader class reads bytes from a stream into a buffer.
 *
 * @author Jun Li (lijun@fmg.cs.ucla.edu)
 * @version $Revision: 1.4 $
 */
public class BytesReader {

    /** 
     * Reads bytes from a stream into a buffer.
     *
     * @param dis    the input stream to read bytes from
     * @param buf    the buffer where the read data to store
     * @param bufLen the size of the buffer
     * @return the number of bytes read from the stream.
     * @exception  java.io.IOException if an error occurs while reading from the stream.
     */
    public static int getFrom(DataInputStream dis, byte[] buf, int bufLen) throws IOException
    {
        int len = dis.readInt();
        if ( bufLen < len ) {
            System.err.println("[BytesReader] Insufficient size buffer");
            System.exit(1);
        }

        int pos = 0;
        while (pos < len) {
            pos += dis.read(buf, pos, len-pos);
        }

        return len;
    }

    /** 
     * Reads bytes from a stream into a buffer.
     *
     * @param dis    the input stream to read bytes from
     * @return a buffer with read bytes from the stream
     * @exception  java.io.IOException if an error occurs while reading from the stream.
     */
    public static byte[] getFrom(DataInputStream dis)  throws IOException {
        byte[] buf;

        int len = dis.readInt();
        buf      = new byte[len];
        int pos = 0;
        while (pos < len) {
            pos += dis.read(buf, pos, len-pos);
        }

        return buf;
    }

}
