/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "FMG_ConductorExt_adaptors_tools_RegEx.h"

#include <regex.h>
#include <stdlib.h>

#define ERR_BUF_SIZE 100
char errbuf[ERR_BUF_SIZE];

/*
 * Class:     FMG_ConductorExt_adaptors_tools_RegEx
 * Method:    doRegComp
 * Signature: (Ljava/lang/String;)I
 */
JNIEXPORT jint JNICALL Java_FMG_ConductorExt_adaptors_tools_RegEx_doRegComp
  (JNIEnv * env, jobject thisObj, jstring regex) {
   regex_t * pregP = (regex_t *) malloc(sizeof(regex_t));

   const char *regex_chars = env->GetStringUTFChars(regex, 0);

   int result = regcomp(pregP, regex_chars, REG_NOSUB);

   env->ReleaseStringUTFChars(regex, regex_chars);

   if (result != 0) {
      // create a string describing the error
      regerror(result, pregP, errbuf, ERR_BUF_SIZE);

      // throw an exception
      jclass excl = env->FindClass(
                      "FMG/ConductorExt/adaptors/tools/RegExException");
      env->ThrowNew(excl, errbuf);
      return 0;
   }

   return (jint) pregP;
}

/*
 * Class:     FMG_ConductorExt_adaptors_tools_RegEx
 * Method:    doRegExec
 * Signature: (ILjava/lang/String;)Z
 */
JNIEXPORT jboolean JNICALL Java_FMG_ConductorExt_adaptors_tools_RegEx_doRegExec
  (JNIEnv *env, jobject thisObj, jint pregP_jint, jstring target) {
   regex_t * pregP = (regex_t *) pregP_jint;

   const char *target_chars = env->GetStringUTFChars(target, 0);

   int result = regexec(pregP, target_chars, 0, NULL, 0);

   env->ReleaseStringUTFChars(target, target_chars);

   if (result == 0) {
      return JNI_TRUE;
   } else if (result == REG_NOMATCH) {
      return JNI_FALSE;
   } else {
      // create a string describing the error
      regerror(result, pregP, errbuf, ERR_BUF_SIZE);

      // throw an exception
      jclass excl = env->FindClass(
                      "FMG/ConductorExt/adaptors/tools/RegExException");
      env->ThrowNew(excl, errbuf);
      return 0;
   }
}

/*
 * Class:     FMG_ConductorExt_adaptors_tools_RegEx
 * Method:    freeMem
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_FMG_ConductorExt_adaptors_tools_RegEx_freeMem
  (JNIEnv *env, jobject thisObj, jint pregP_jint) {
   regex_t * pregP = (regex_t *) pregP_jint;

   free(pregP);
}
