/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <getopt.h>
#include <errno.h>
#include <fcntl.h>
#include <ctype.h>
#include <sys/time.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include <pwd.h>
#include "FMG_ConductorExt_adaptors_tools_image_RenderAsciiImage.h"
#include <aalib.h>

JNIEXPORT jcharArray JNICALL Java_FMG_ConductorExt_adaptors_tools_image_RenderAsciiImage_RenderAscii
(JNIEnv *env, jclass thisClass, jintArray intArray) {
  aa_context *context;
  jcharArray outArray;
  struct aa_renderparams *ascii_parms; // = { 50, 10, 10, AA_FLOYD_S, 0, 0};
  struct aa_hardware_params ascii;
  jint *intArrayElems;
  jint length;
  long width, height;
  unsigned char *image;
  jchar *imgarray;
  int ctr = 0;
  int r,g,b;
  jboolean isCopy = JNI_FALSE;
  printf("Entered library.\n");  
  memcpy(&ascii,&aa_defparams,sizeof(struct aa_hardware_params));
  ascii_parms = aa_getrenderparams();
  length = env->GetArrayLength(intArray);
  printf("length is: %d\n",length);  

  intArrayElems = env->GetIntArrayElements(intArray, &isCopy);
  width = intArrayElems[length-2];
  height = intArrayElems[length-1];
  printf("height: %d width %d\n",height, width);  
  ascii.width = (int) width;
  ascii.height = (int) height;
  context = aa_init(&mem_d, &ascii, NULL);
  printf("got context %x\n",&context);  
  image = (unsigned char *) calloc(length-2, sizeof(char));
  length -=2;


  for(int i=0; i < length; i++)
    image[i] = (char) (intArrayElems[i] & 0xff);

  printf("copying memory...\n");    

  memcpy(aa_image(context),image,length);
  printf("rendering ascii...\n");  
  aa_render(context,ascii_parms,0,0,width,height);
  printf("rendered.\n");  
  height = aa_scrheight(context)/2;
  width = aa_scrwidth(context)/2;
  imgarray = (jchar *)calloc(height*(width+1), sizeof(jchar));
  printf("building char array\n");  

  ctr = 0;
  for (int y = 0; y < height; y++) {
    for (int x = 0; x < width; x++) {
      imgarray[ctr++] = context->textbuffer[x + y * width];
    }
  }

  length = height*width;
  aa_close(context);
  outArray = env->NewCharArray(length);
  env->SetCharArrayRegion(outArray,0,length, imgarray);
  free(image);
  free(imgarray);
  env->ReleaseIntArrayElements(intArray, intArrayElems, 0);
  return outArray;

}




