/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <netinet/in.h>
#include <netinet/ip.h>
#include <sys/socket.h>
#include "FMG_Conductor_net_SocketOps.h"

/*
 * Class:     FMG_Conductor_net_SocketOps
 * Method:    shutdownFD
 * Signature: (II)V
 */
JNIEXPORT void JNICALL Java_FMG_Conductor_net_SocketOps_shutdownFD
  (JNIEnv *env, jclass thisClass, jint fd, jint how) {
   if (shutdown((int) fd, (int) how)<0) {
      jclass excl = env->FindClass("java/io/IOException");
      env->ThrowNew(excl, "shutdown() failed");
      return;
   }
}

/*
 * Class:     FMG_Conductor_net_SocketOps
 * Method:    setInputBuffer
 * Signature: (II)V
 */
JNIEXPORT void JNICALL Java_FMG_Conductor_net_SocketOps_setInputBuffer
  (JNIEnv *env, jclass thisClass, jint fd, jint size) {
   int bufSize = size;  /* Make sure we get the operand type right */

   if (setsockopt((int) fd, SOL_SOCKET, SO_RCVBUF, &bufSize, sizeof(int))<0) {
      jclass excl = env->FindClass("java/io/IOException");
      env->ThrowNew(excl, "setsockopt(...SO_RCVBUF...) failed");
      return;
   }
}

/*
 * Class:     FMG_Conductor_net_SocketOps
 * Method:    setOutputBuffer
 * Signature: (II)V
 */
JNIEXPORT void JNICALL Java_FMG_Conductor_net_SocketOps_setOutputBuffer
  (JNIEnv *env, jclass thisClass, jint fd, jint size) {
   int bufSize = size;  /* Make sure we get the operand type right */

   if (setsockopt((int) fd, SOL_SOCKET, SO_SNDBUF, &bufSize, sizeof(int))<0) {
      jclass excl = env->FindClass("java/io/IOException");
      env->ThrowNew(excl, "setsockopt(...SO_SNDBUF...) failed");
      return;
   }
}

/*
 * Class:     FMG_Conductor_net_SocketOps
 * Method:    setPriority
 * Signature: (II)V
 */
JNIEXPORT void JNICALL Java_FMG_Conductor_net_SocketOps_setPriority
  (JNIEnv *env, jclass thisClass, jint fd, jint pri) {
   int priority = pri;  /* Make sure we get the operand type right */

   if (setsockopt((int) fd, SOL_SOCKET, SO_PRIORITY, 
                                       &priority, sizeof(int))<0) {
      jclass excl = env->FindClass("java/io/IOException");
      env->ThrowNew(excl, "setsockopt(...SO_PRIORITY...) failed");
      return;
   }
}

/*
 * Class:     FMG_Conductor_net_SocketOps
 * Method:    setTOS
 * Signature: (II)V
 */
JNIEXPORT void JNICALL Java_FMG_Conductor_net_SocketOps_setTOS
  (JNIEnv *env, jclass thisClass, jint fd, jint typeOfService) {
   int tos = typeOfService;  /* Make sure we get the operand type right */

   tos = IPTOS_TOS(tos);

   if (setsockopt((int) fd, SOL_IP, IP_TOS, &tos, sizeof(int))<0) {
      jclass excl = env->FindClass("java/io/IOException");
      env->ThrowNew(excl, "setsockopt(...SO_TOS...) failed");
      return;
   }
}

/*
 * Class:     FMG_Conductor_net_SocketOps
 * Method:    getFD
 * Signature: (Ljava/io/FileDescriptor;)I
 */
JNIEXPORT jint JNICALL Java_FMG_Conductor_net_SocketOps_getFD__Ljava_io_FileDescriptor_2
  (JNIEnv *env, jclass thisClass, jobject fdes) {
    jclass fdesclass = env->GetObjectClass(fdes);
    jfieldID fdID = env->GetFieldID(fdesclass, "fd", "I");
#ifdef JDK1_2
    return env->GetIntField(fdes, fdID);
#else
    return env->GetIntField(fdes, fdID) - 1;
#endif
}

/*
 * Class:     FMG_Conductor_net_SocketOps
 * Method:    getFD
 * Signature: (Ljava/net/Socket;)I
 */
JNIEXPORT jint JNICALL Java_FMG_Conductor_net_SocketOps_getFD__Ljava_net_Socket_2
  (JNIEnv *env, jclass thisClass, jobject socket) {
   // Get the SocketImpl object
   jclass sockclass = env->GetObjectClass(socket);
   jfieldID implID = env->GetFieldID(sockclass, "impl",
                                     "Ljava/net/SocketImpl;");
   jobject impl = env->GetObjectField(socket, implID);

   // Get the FileDescriptor object
   jclass implclass = env->GetObjectClass(impl);
   jfieldID filedesID = env->GetFieldID(implclass, "fd",
                                        "Ljava/io/FileDescriptor;");
   jobject fdesobj = env->GetObjectField(impl, filedesID);

   // call getFD(FileDescriptor)
   return Java_FMG_Conductor_net_SocketOps_getFD__Ljava_io_FileDescriptor_2(
                                                     env, thisClass, fdesobj);
}

/*
 * Class:     FMG_Conductor_net_SocketOps
 * Method:    getFD
 * Signature: (Ljava/net/DatagramSocket;)I
 */
JNIEXPORT jint JNICALL Java_FMG_Conductor_net_SocketOps_getFD__Ljava_net_DatagramSocket_2
  (JNIEnv *env, jclass thisClass, jobject socket) {
   // Get the SocketImpl object
   jclass sockclass = env->GetObjectClass(socket);
   jfieldID implID = env->GetFieldID(sockclass, "impl",
       "Ljava/net/DatagramSocketImpl;");
   jobject impl = env->GetObjectField(socket, implID);

   // Get the FileDescriptor object
   jclass implclass = env->GetObjectClass(impl);
   jfieldID filedesID = env->GetFieldID(implclass, "fd",
       "Ljava/io/FileDescriptor;");
   jobject fdesobj = env->GetObjectField(impl, filedesID);

   // call getFD(FileDescriptor)
   return Java_FMG_Conductor_net_SocketOps_getFD__Ljava_io_FileDescriptor_2(
                                                     env, thisClass, fdesobj);
}

/*
 * Class:     FMG_Conductor_net_SocketOps
 * Method:    getFD
 * Signature: (Ljava/io/FileInputStream;)I
 */
JNIEXPORT jint JNICALL Java_FMG_Conductor_net_SocketOps_getFD__Ljava_io_FileInputStream_2
  (JNIEnv *env, jclass thisClass, jobject is) {
   // get the FileDescriptor object for this input stream
   jclass isclass = env->GetObjectClass(is);
   jfieldID filedesID = env->GetFieldID(isclass, "fd",
                                        "Ljava/io/FileDescriptor;");
   jobject fdesobj = env->GetObjectField(is, filedesID);

   // call getFD(FileDescriptor)
   return Java_FMG_Conductor_net_SocketOps_getFD__Ljava_io_FileDescriptor_2(
                                                     env, thisClass, fdesobj);
}

/*
 * Class:     FMG_Conductor_net_SocketOps
 * Method:    getFD
 * Signature: (Ljava/io/FileOutputStream;)I
 */
JNIEXPORT jint JNICALL Java_FMG_Conductor_net_SocketOps_getFD__Ljava_io_FileOutputStream_2
  (JNIEnv *env, jclass thisClass, jobject os) {
   // get the FileDescriptor object for this output stream
   jclass osclass = env->GetObjectClass(os);
   jfieldID filedesID = env->GetFieldID(osclass, "fd",
                                        "Ljava/io/FileDescriptor;");
   jobject fdesobj = env->GetObjectField(os, filedesID);

   // call getFD(FileDescriptor)
   return Java_FMG_Conductor_net_SocketOps_getFD__Ljava_io_FileDescriptor_2(
                                                     env, thisClass, fdesobj);
}
